<?php

namespace FiloBlu\Flow\Model\Actions\From;

class Addtocategory {

    
    CONST ENTITY_TYPE_CODE = 'catalog_product';

    /**
     * Categories text-path to ID hash.
     *
     * @var array
     */
    protected $_categories = array();
    /**
     * Categories text-path to ID hash with roots checking.
     *
     * @var array
     */
    protected $_categoriesWithRoots = array();
    protected $_namesPaths = null;
    protected $_helper_product;
    protected $_configurable_type;
    protected $_category_factory;
    protected $_product_factory;


    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Product $helper_product,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        \Magento\Catalog\Model\CategoryFactory $categoryFactory,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable $configurableProduct,
        array $data = []
    ) {
        $this->_helper_product = $helper_product;
        $this->_category_factory = $categoryFactory;
        $this->_product_factory = $productFactory;
        $this->_configurable_type = $configurableProduct;
        $this->_initCategories();
    }

    public function process($product, $value, $storeId) {

        if (empty($value)) {
            return true;
        }

        if(empty($this->_categories)){
            $this->_initCategories();
        }

        $position = 0;
        $category = null;
        $originalValue = $value;
        
        $_category_tree = array();

        // controllo se si tratta di più categorie con separatore |
        $_new_cat_ids = '';
        if(strrpos($originalValue, '|')) {
            $_new_cat_ids = explode( '|', $originalValue);
        }
        // gestione posizione
        /*
        $values = explode('#', $value);
        if (count($values) == 2) {
            $value = $values[0];
            $position = $values[1];
        }
        else {
            $value = $values[0];
        }
        */

        // position must be set even if product already exists (EG: positions update)

        if(is_array($_new_cat_ids) && count($_new_cat_ids) > 1){
            foreach($_new_cat_ids as $_cat_id) {
                if (is_numeric($_cat_id)) {
                    $category = $this->_category_factory->create()->load($_cat_id);
                    if  (!is_object($category) || $category->getId() <= 0) {
                         $this->_logger->info("Category ID {$value} does not exist - SKIPPED ");
                    } else {
                        $_category_tree =  array_merge( explode('/', $category->getPath()), $_category_tree );
                    }
                }
            }
        } else {

            if (is_numeric($value)) {
                $category = $this->_category_factory->create()->load($value);
            } else {
                $category = \Magento\Framework\App\ObjectManager::getInstance()->create('\Magento\Catalog\Model\ResourceModel\Category\Collection')
                    ->addAttributeToFilter('path', array('like' => $this->getCategoryIdsPathByNamesPath($value, false, true)))
                    ->getFirstItem();
                if (is_object($category) && $category->getId() > 0) {
                    $category = $category->load($category->getId());
                }
                if(is_numeric($catId = $this->getIdFromPath($value))){
                    $category = $this->_category_factory->create()->load($catId);
                }
            }

            if  (!is_object($category) || $category->getId() <= 0) {
                return "Category ID {$value} does not exist";
            }
            $path = $category->getPath();
            $_category_tree = explode('/', $path);
        }

        // mettere in configurazione se le categorie vanno in sostituzione o in update
        // di default in sostituzione

        $_category_update = false;
        $merged_category = array();
        $old_categories = $product->getCategoryIds();
        if ($_category_update && count($old_categories))
            $merged_category = array_merge($old_categories,$_category_tree);
        else
            $merged_category = $_category_tree;

        $merged_category = array_unique($merged_category);
        $merged_category = array_diff($merged_category, array(1, 2)); // tolgo root category ID

        //mettere in configurazione se salvare le categorie solo sui configurabili o anche sui semplici
        $_only_configurable_category = false;
        if ( $_only_configurable_category ) {

            if ($product->getData('visibility') != '1') {
                $product->setCategoryIds($merged_category);
                $product->save();
            }
            else {
                $product->setCategoryIds(array());
                $product->save();
            }
        } else {
            $product->setCategoryIds($merged_category);
            $product->save();
        }


        if ($product->getTypeId() == \Magento\Catalog\Model\Product\Type::DEFAULT_TYPE) {
            $parentIds = $this->_configurable_type->getParentIdsByChild($product->getId());
            if (count($parentIds) <= 0)
                return true;

            foreach ($parentIds as $parentId) {
                $parent_product = $this->_product_factory->create()->load($parentId);
                // Sostituzione delle categorie
                // $merged_parent_category = array_unique(array_merge($merged_category,$parent_product->getCategoryIds()));
                $parent_product->setCategoryIds($merged_category)->save();
            }

        }

        return true;
    }

    /**
     * Finds a subcategory id from a path string
     *
     * @param $string
     * @return bool
     */
    public function getIdFromPath($string)
    {
        if (in_array(strtoupper($string), array_keys($this->_categories))) {
            return $this->_categories[strtoupper($string)];
        }
        return false;
    }
    /**
     * Returns all valid path strings
     *
     * @return array
     */
    public function getAllPaths()
    {
        return array_keys($this->_categories);
    }
    /**
     * Initialize categories text-path to ID hash.
     *
     * @return Mage_ImportExport_Model_Import_Entity_Product
     */
    protected function _initCategories()
    {
        $collection = \Magento\Framework\App\ObjectManager::getInstance()->create('\Magento\Catalog\Model\ResourceModel\Category\Collection');
        $collection->setStore(0);
        $collection->addNameToResult();

        foreach ($collection as $category) {
            $structure = explode('/', $category->getPath());
            $pathSize = count($structure);
            if ($pathSize > 1) {
                $path = array();
                for ($i = 1; $i < $pathSize; $i++) {
                    $path[] = $collection->getItemById($structure[$i])->getName();
                }
                $rootCategoryName = array_shift($path);
                if (!isset($this->_categoriesWithRoots[$rootCategoryName])) {
                    $this->_categoriesWithRoots[$rootCategoryName] = array();
                }
                $index = implode('/', $path);
                $index = strtoupper($index);
                $this->_categoriesWithRoots[$rootCategoryName][$index] = $category->getId();
                if ($pathSize > 2) {
                    $this->_categories[$index] = $category->getId();
                }
            }
        }
        return $this;
    }


    protected function getCategoryIdsPathByNamesPath($names_path, $includeRoot = true, $addLike = false) {

        $paths = $this->getCategoryNamesPaths();
        if (!isset($paths['name2id']) || count($paths['name2id']) <= 0) {
            return false;
        }

        $name2id = $paths['name2id'];

        $pathParts = explode('/', $names_path);
        if (!is_array($pathParts) || count($pathParts) <= 0) {
            return false;
        }

        $ids_path_result = array();
        foreach ($pathParts as $singlePathName) {
            if (!isset($name2id[trim($singlePathName)])) {
                return false;
            }

            $ids_path_result[] = $name2id[$singlePathName];
        }

        $ids_path = implode('/', $ids_path_result);
        if ($includeRoot) {
            //$ids_path = Mage::app()->getDefaultStoreView()->getRootCategoryId() . '/' . $ids_path;
        }

        if ($addLike) {
            $ids_path = '%' . $ids_path;
        }
        return $ids_path;
    }

    protected function getCategoryNamesPaths() {
        
        if (is_null($this->_namesPaths)) {
            
            $this->_namesPaths = array('id2name' => array(),'name2id' => array());
            $categoriesCollection = \Magento\Framework\App\ObjectManager::getInstance()->create('\Magento\Catalog\Model\ResourceModel\Category\Collection')
                ->setStore(0)
                ->addAttributeToSelect('*');

            foreach ($categoriesCollection as $singleCategory) {
                $categoryName = trim($singleCategory->getName());

                $this->_namesPaths['id2name'][$singleCategory->getId()] = $categoryName;
                $this->_namesPaths['name2id'][$categoryName] = $singleCategory->getId();
            }
            
        }

        return $this->_namesPaths;
    }

}
